import os
from unittest import TestCase
from mailosaur import MailosaurClient

class UsageTest(TestCase):
    @classmethod
    def setUpClass(self):
        api_key = os.getenv('MAILOSAUR_API_KEY')
        base_url = os.getenv('MAILOSAUR_BASE_URL')

        if api_key is None:
            raise Exception("Missing necessary environment variables - refer to README.md")

        self.client = MailosaurClient(api_key, base_url)

    def test_account_limits(self):
        result = self.client.usage.limits()
        self.assertIsNotNone(result.servers)
        self.assertIsNotNone(result.users)
        self.assertIsNotNone(result.email)
        self.assertIsNotNone(result.sms)

        self.assertTrue(result.servers.limit > 0)
        self.assertTrue(result.users.limit > 0)
        self.assertTrue(result.email.limit > 0)
        self.assertTrue(result.sms.limit > 0)

    def test_transactions_list(self):
        result = self.client.usage.transactions()
        self.assertTrue(len(result.items) > 1)
        self.assertIsNotNone(result.items[0].timestamp)
        self.assertIsNotNone(result.items[0].email)
        self.assertIsNotNone(result.items[0].sms)



import time
from datetime import datetime, timedelta
from ..models import MessageListResult
from ..models import Message
from ..models import PreviewListResult
from ..models import MailosaurException


class MessagesOperations(object):
    """MessagesOperations operations.
    """

    def __init__(self, session, base_url, handle_http_error):
        self.session = session
        self.base_url = base_url
        self.handle_http_error = handle_http_error

    def get(self, server, criteria, timeout=10000, received_after=(datetime.today() - timedelta(hours=1)), dir=None):
        """Retrieve a message using search criteria.

        Returns as soon as a message matching the specified search criteria is
        found. This is the most efficient method of looking up a message.

        :param server: The identifier of the server hosting the message.
        :type server: str
        :param criteria: The search criteria to use in order to find a match.
        :type criteria: ~mailosaur.models.SearchCriteria      
        :param timeout: Specify how long to wait for a matching result (in milliseconds).
        :type timeout: int
        :param received_after: Limits results to only messages received after this date/time.
        :type received_after: datetime
        :param dir: Optionally limits results based on the direction (`Sent` or `Received`),
         with the default being `Received`.
        :type dir: str
        :return: Message
        :rtype: ~mailosaur.models.Message
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        # Defaults timeout to 10s, receivedAfter to 1h
        if len(server) != 8:
            raise MailosaurException(
                "Must provide a valid Server ID.", "invalid_request")

        result = self.search(server, criteria, 0, 1,
                             timeout, received_after, True, dir)
        return self.get_by_id(result.items[0].id)

    def get_by_id(self, id):
        """Retrieve a message.

        Retrieves the detail for a single email message. Simply supply the
        unique identifier for the required message.

        :param id: The identifier of the email message to be retrieved.
        :type id: str        
        :return: Message
        :rtype: ~mailosaur.models.Message
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/%s" % (self.base_url, id)
        response = self.session.get(url)

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return Message(data)

    def delete(self, id):
        """Delete a message.

        Permanently deletes a message. This operation cannot be undone. Also
        deletes any attachments related to the message.

        :param id: The identifier of the message to be deleted.
        :type id: str        
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/%s" % (self.base_url, id)
        response = self.session.delete(url)

        if response.status_code not in [204]:
            self.handle_http_error(response)
            return

    def list(self, server, page=None, items_per_page=None, received_after=None, dir=None):
        """List all messages.

        Returns a list of your messages in summary form. The summaries are
        returned sorted by received date, with the most recently-received
        messages appearing first.

        :param server: The identifier of the server hosting the messages.
        :type server: str
        :param page: Used in conjunction with `itemsPerPage` to support
         pagination.
        :type page: int
        :param items_per_page: A limit on the number of results to be returned
         per page. Can be set between 1 and 1000 items, the default is 50.
        :type items_per_page: int   
        :param received_after: Limits results to only messages received after this date/time.
        :type received_after: datetime
        :param dir: Optionally limits results based on the direction (`Sent` or `Received`),
         with the default being `Received`.
        :type dir: str
        :return: MessageListResult
        :rtype: ~mailosaur.models.MessageListResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages" % (self.base_url)

        if received_after is not None:
            received_after = received_after.astimezone().replace(microsecond=0).isoformat()

        params = {'server': server, 'page': page,
                  'itemsPerPage': items_per_page, 'receivedAfter': received_after, 'dir': dir}
        response = self.session.get(url, params=params)

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return MessageListResult(data)

    def delete_all(self, server):
        """Delete all messages.

        Permanently deletes all messages held by the specified server. This
        operation cannot be undone. Also deletes any attachments related to
        each message.

        :param server: The identifier of the server to be emptied.
        :type server: str        
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages" % (self.base_url)
        params = {'server': server}
        response = self.session.delete(url, params=params)

        if response.status_code not in [204]:
            self.handle_http_error(response)
            return

    def search(self, server, criteria, page=None, items_per_page=None, timeout=None, received_after=None, error_on_timeout=True, dir=None):
        """Search for messages.

        Returns a list of messages matching the specified search criteria, in
        summary form. The messages are returned sorted by received date, with
        the most recently-received messages appearing first.

        :param server: The identifier of the server hosting the messages.
        :type server: str
        :param criteria: The search criteria to match results against.
        :type criteria: ~mailosaur.models.SearchCriteria
        :param page: Used in conjunction with `itemsPerPage` to support
         pagination.
        :type page: int
        :param items_per_page: A limit on the number of results to be returned
         per page. Can be set between 1 and 1000 items, the default is 50.
        :type items_per_page: int
        :param timeout: Specify how long to wait for a matching result (in milliseconds).
        :type timeout: int
        :param received_after: Limits results to only messages received after this date/time.
        :type received_after: datetime
        :param error_on_timeout: When set to false, an error will not be throw if timeout 
         is reached (default: true).
        :type error_on_timeout: bool
        :param dir: Optionally limits results based on the direction (`Sent` or `Received`),
         with the default being `Received`.
        :type dir: str
        :return: MessageListResult
        :rtype: ~mailosaur.models.MessageListResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/search" % (self.base_url)

        if received_after is not None:
            received_after = received_after.astimezone().replace(microsecond=0).isoformat()

        params = {'server': server, 'page': page,
                  'itemsPerPage': items_per_page, 'receivedAfter': received_after, 'dir': dir}

        poll_count = 0
        start_time = datetime.today()

        while True:
            response = self.session.post(
                url, params=params, json=criteria.to_json())

            if response.status_code not in [200]:
                self.handle_http_error(response)
                return

            data = response.json()

            result = MessageListResult(data)

            if timeout is None or timeout == 0 or len(result.items) != 0:
                return result

            # List conversion necessary for Python 3 compatibility
            # https://stackoverflow.com/questions/36982858/object-of-type-map-has-no-len-in-python-3
            delay_pattern = list(
                map(int, (response.headers.get('x-ms-delay') or '1000').split(',')))

            delay = delay_pattern[len(
                delay_pattern) - 1] if poll_count >= len(delay_pattern) else delay_pattern[poll_count]

            poll_count += 1

            # Stop if timeout will be exceeded
            if ((1000 * (datetime.today() - start_time).total_seconds()) + delay) > timeout:
                if not error_on_timeout:
                    return result
                else:
                    raise MailosaurException(
                        "No matching messages found in time. By default, only messages received in the last hour are checked (use receivedAfter to override this). The search criteria used for this query was [%s] which timed out after %sms" % (criteria.to_json(), timeout), "search_timeout")

            time.sleep(delay / 1000)

    def create(self, server, options):
        """Create a message.

        Creates a new message that can be sent to a verified email address. This is
        useful in scenarios where you want an email to trigger a workflow in your
        product

        :param server: The identifier of the server to create the message in.
        :type server: str
        :param options: The options with which to create the message.
        :type options: ~mailosaur.models.MessageCreateOptions
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages" % (self.base_url)
        params = {'server': server}
        response = self.session.post(
            url, params=params, json=options.to_json())

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return Message(data)

    def forward(self, id, options):
        """Forward an email.

        Forwards the specified email to a verified email address.

        :param id: The identifier of the email to forward.
        :type id: str
        :param options: The options with which to forward the email.
        :type options: ~mailosaur.models.MessageForwardOptions
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/%s/forward" % (self.base_url, id)
        response = self.session.post(url, json=options.to_json())

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return Message(data)

    def reply(self, id, options):
        """Reply to an email.

        Sends a reply to the specified email. This is useful for when simulating a user
        replying to one of your emails.

        :param id: The identifier of the email to reply to.
        :type id: str
        :param options: The options with which to reply to the email.
        :type options: ~mailosaur.models.MessageReplyOptions
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/%s/reply" % (self.base_url, id)
        response = self.session.post(url, json=options.to_json())

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return Message(data)

    def generate_previews(self, id, options):
        """Generate email previews.

        Generates screenshots of an email rendered in the specified email clients.

        :param id: The identifier of the email to preview.
        :type id: str
        :param options: The options with which to generate previews.
        :type options: ~mailosaur.models.PreviewRequestOptions
        :return: PreviewListResult
        :rtype: ~mailosaur.models.PreviewListResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/messages/%s/previews" % (self.base_url, id)
        response = self.session.post(url, json=options.to_json())

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return PreviewListResult(data)



class Preview(object):
    """Describes an email preview.

    :param id: Unique identifier for the email preview.
    :type id: str

    :param email_client: The email client the preview was generated with.
    :type email_client: str

    :param disable_images: True if images were disabled in the preview.
    :type disable_images: bool
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.id = data.get('id', None)
        self.email_client = data.get('emailClient', None)
        self.disable_images = data.get('disableImages', None)

    def to_json(self):
        return {
            'id': self.id,
            'emailClient': self.email_client,
            'disableImages': self.disable_images
        }



import os
import time
import string
import random
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage


class Mailer(object):
    html = open(os.path.join(os.path.dirname(__file__),
                'resources', 'testEmail.html'), 'r').read()
    text = open(os.path.join(os.path.dirname(__file__),
                'resources', 'testEmail.txt'), 'r').read()
    verified_domain = os.getenv('MAILOSAUR_VERIFIED_DOMAIN') or "mailosaur.net"

    @staticmethod
    def send_emails(client, server, quantity):
        for i in range(0, quantity):
            Mailer.send_email(client, server)

    @staticmethod
    def send_email(client, server, send_to_address=None):
        host = os.getenv('MAILOSAUR_SMTP_HOST', 'mailosaur.net')
        port = os.getenv('MAILOSAUR_SMTP_PORT', '25')

        message = MIMEMultipart('related')

        randomString = ''.join(random.choice(
            string.ascii_uppercase + string.ascii_lowercase) for _ in range(10))

        message['Subject'] = "%s subject" % (randomString)

        random_to_address = send_to_address
        if (random_to_address == None):
            random_to_address = client.servers.generate_email_address(server)

        message['From'] = "%s %s <%s@%s>" % (randomString,
                                             randomString, randomString, Mailer.verified_domain)
        message['To'] = "%s %s <%s>" % (
            randomString, randomString, random_to_address)

        alt = MIMEMultipart('alternative')
        message.attach(alt)

        # Text body
        alt.attach(MIMEText(Mailer.text.replace(
            "REPLACED_DURING_TEST", randomString)))

        # Html body
        alt.attach(MIMEText(Mailer.html.replace(
            "REPLACED_DURING_TEST", randomString), 'html'))

        fp = open(os.path.join(os.path.dirname(
            __file__), 'resources', 'cat.png'), 'rb')
        img = MIMEImage(fp.read())
        img.add_header('Content-ID', 'ii_1435fadb31d523f6')
        img.add_header('Content-Disposition', 'inline', filename='cat.png')
        message.attach(img)
        fp.close()

        fp = open(os.path.join(os.path.dirname(
            __file__), 'resources', 'dog.png'), 'rb')
        img = MIMEImage(fp.read())
        img.add_header('Content-Disposition', 'attachment', filename='dog.png')
        message.attach(img)
        fp.close()

        smtp = smtplib.SMTP()
        smtp.connect(host, port)
        smtp.sendmail(message['From'], message['To'], message.as_string())
        smtp.quit()



from .preview_email_client import PreviewEmailClient


class PreviewEmailClientListResult(object):
    """A list of available email clients with which to generate email previews.

    :param items: A list of available email clients.
    :type items: list[~mailosaur.models.PreviewEmailClient]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.items = [PreviewEmailClient(i) for i in data.get('items', None)]



from .message_header import MessageHeader
from .message_address import MessageAddress


class Metadata(object):
    """Advanced use case content related to the message.

    :param headers: Email headers.
    :type headers: list[~mailosaur.models.MessageHeader]
    :param mail_from:
    :type mail_from: str
    :param rcpt_to:
    :type rcpt_to: list[~mailosaur.models.MessageAddress]
    :param ehlo:
    :type ehlo: str
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.headers = [MessageHeader(i) for i in data.get('headers', [])]
        self.ehlo = data.get('ehlo', None)
        self.mail_from = data.get('mailFrom', None)
        self.rcpt_to = [MessageAddress(i) for i in data.get('rcptTo', [])]



import dateutil.parser


class OtpResult(object):
    """OtpResult.

    :param code: The current one-time password.
    :type code: str
    :param expires: The expiry date/time of the current one-time password.
    :type expires: datetime
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.code = data.get('code', None)
        self.expires = dateutil.parser.parse(data.get('expires', None))



import dateutil.parser
from .message_address import MessageAddress


class MessageSummary(object):
    """MessageSummary.

    :param id:
    :type id: str
    :param type:
    :type type: str
    :param server:
    :type server: str
    :param sender:
    :type sender: list[~mailosaur.models.MessageAddress]
    :param to:
    :type to: list[~mailosaur.models.MessageAddress]
    :param cc:
    :type cc: list[~mailosaur.models.MessageAddress]
    :param bcc:
    :type bcc: list[~mailosaur.models.MessageAddress]
    :param received:
    :type received: datetime
    :param subject:
    :type subject: str
    :param summary:
    :type summary: str
    :param attachments:
    :type attachments: int
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.id = data.get('id', None)
        self.type = data.get('type', None)
        self.server = data.get('server', None)
        self.sender = [MessageAddress(i) for i in data.get('from', [])]
        self.to = [MessageAddress(i) for i in data.get('to', [])]
        self.cc = [MessageAddress(i) for i in data.get('cc', [])]
        self.bcc = [MessageAddress(i) for i in data.get('bcc', [])]
        self.received = dateutil.parser.parse(data.get('received', None))
        self.subject = data.get('subject', None)
        self.summary = data.get('summary', None)
        self.attachments = data.get('attachments', 0)



class PreviewRequest(object):
    """Describes an email preview request.

    :param email_client: The email client you wish to generate a preview for.
    :type email_client: str

    :param disable_images: If true, images will be disabled (only if supported by the client).
    :type disable_images: bool
    """

    def __init__(self, email_client, disable_images=False):
        self.email_client = email_client
        self.disable_images = disable_images

    def to_json(self):
        return {
            'emailClient': self.email_client,
            'disableImages': self.disable_images
        }



from .spam_assassin_rule import SpamAssassinRule
from .result_enum import ResultEnum

class SpamAssassinResult(object):
    """SpamAssassinResult.

    :param score:
    :type score: float
    :param result:
    :type result: ~mailosaur.models.ResultEnum
    :param rules:
    :type rules: list[~mailosaur.models.SpamAssassinRule]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.score = data.get('score', 0.0)
        self.result = ResultEnum(data.get('result', None))
        self.rules = [SpamAssassinRule(i) for i in data.get('rules', None)]



import os
import random
import string

from mailosaur.models.otp_result import OtpResult
from ..models import DeviceListResult
from ..models import Device
from ..models import OtpResult
from ..models import MailosaurException


class DevicesOperations(object):
    """DevicesOperations operations.
    """

    def __init__(self, session, base_url, handle_http_error):
        self.session = session
        self.base_url = base_url
        self.handle_http_error = handle_http_error

    def generate_email_address(self, server):
        host = os.getenv('MAILOSAUR_SMTP_HOST', 'mailosaur.net')
        randomString = ''.join(random.choice(
            string.ascii_uppercase + string.digits) for _ in range(10))
        return "%s@%s.%s" % (randomString, server, host)

    def list(self):
        """List all devices.

        Returns a list of your virtual security devices.

        :return: DeviceListResult
        :rtype: ~mailosaur.models.DeviceListResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/devices" % (self.base_url)
        response = self.session.get(url)

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return DeviceListResult(data)

    def create(self, device_create_options):
        """Create a device.

        Creates a new virtual security device and returns it.

        :param device_create_options:
        :type device_create_options: ~mailosaur.models.DeviceCreateOptions
        :return: Device
        :rtype: ~mailosaur.models.Device
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/devices" % (self.base_url)
        response = self.session.post(url, json=device_create_options.to_json())

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return Device(data)

    def otp(self, query):
        """Retrieves the current one-time password for a saved device, or given base32-encoded shared secret.

        Retrieves the detail for a single server. Simply supply the unique
        identifier for the required server.

        :param query: Either the unique identifier of the device, or a base32-encoded shared secret.
        :type query: str        
        :return: OtpResult
        :rtype: ~mailosaur.models.OtpResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        if "-" in query:
            url = "%sapi/devices/%s/otp" % (self.base_url, query)
            response = self.session.get(url)

            if response.status_code not in [200]:
                self.handle_http_error(response)
                return

            data = response.json()

            return OtpResult(data)

        url = "%sapi/devices/otp" % (self.base_url)
        response = self.session.post(url, json={'sharedSecret': query})

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return OtpResult(data)

    def delete(
            self, id):
        """Delete a device.

        Permanently delete a virtual security device. This operation cannot be undone.

        :param id: The identifier of the device to be deleted.
        :type id: str        
        :return: None
        :rtype: None
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/devices/%s" % (self.base_url, id)
        response = self.session.delete(url)

        if response.status_code not in [204]:
            self.handle_http_error(response)
            return



class UsageAccountLimit(object):
    """UsageAccountLimit.

    :param limit:
    :type limit: int
    :param current:
    :type current: int
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.limit = data.get('limit', None)
        self.current = data.get('current', None)



class DeviceListResult(object):
    """The result of a device listing request.

    :param items: The individual devices forming the result.
    :type items: list[~mailosaur.models.Device]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.items = data.get('items', None)



class DeviceCreateOptions(object):
    """DeviceCreateOptions.

    :param name: A name used to identify the device.
    :type name: str
    :param shared_secret: The base32-encoded shared secret for this device.
    :type shared_secret: str
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.name = data.get('name', None)
        self.shared_secret = data.get('shared_secret', None)

    def to_json(self):
        return {
            'name': self.name,
            'sharedSecret': self.shared_secret
        }



from ..models import PreviewEmailClientListResult


class PreviewsOperations(object):
    """PreviewsOperations operations.
    """

    def __init__(self, session, base_url, handle_http_error):
        self.session = session
        self.base_url = base_url
        self.handle_http_error = handle_http_error

    def list_email_clients(self):
        """List all email clients that can be used to generate email previews.

        Returns a list of available email clients.

        :return: PreviewEmailClientListResult
        :rtype: ~mailosaur.models.PreviewEmailClientListResult
        :raises:
         :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/previews/clients" % (self.base_url)
        response = self.session.get(url)

        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return PreviewEmailClientListResult(data)



from .usage_account_limit import UsageAccountLimit

class UsageAccountLimits(object):
    """UsageAccountLimits.

    :param servers:
    :type servers: ~mailosaur.models.UsageAccountLimit
    :param users:
    :type users: ~mailosaur.models.UsageAccountLimit
    :param email:
    :type email: ~mailosaur.models.UsageAccountLimit
    :param sms:
    :type sms: ~mailosaur.models.UsageAccountLimit
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.servers = UsageAccountLimit(data.get('servers', None))
        self.users = UsageAccountLimit(data.get('users', None))
        self.email = UsageAccountLimit(data.get('email', None))
        self.sms = UsageAccountLimit(data.get('sms', None))




class Content(object):
    """Content.

    :param embed:
    :type embed: boolean
    :param iframe:
    :type iframe: boolean
    :param object:
    :type object: boolean
    :param script:
    :type script: boolean
    :param short_urls:
    :type short_urls: boolean
    :param text_size:
    :type text_size: int
    :param total_size:
    :type total_size: int
    :param missing_alt:
    :type missing_alt: boolean
    :param missing_list_unsubscribe:
    :type missing_list_unsubscribe: boolean
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.embed = data.get('embed', None)
        self.iframe = data.get('iframe', None)
        self.object = data.get('object', None)
        self.script = data.get('script', None)
        self.short_urls = data.get('shortUrls', None)
        self.text_size = data.get('textSize', None)
        self.total_size = data.get('totalSize', None)
        self.missing_alt = data.get('missingAlt', None)
        self.missing_list_unsubscribe = data.get('missingListUnsubscribe', None)


import json


class PreviewRequestOptions(object):
    """PreviewRequestOptions.

    :param previews: The list of email preview requests.
    :type previews: list[~mailosaur.models.PreviewRequest]
    """

    def __init__(self, previews=None):
        self.previews = previews

    def to_json(self):
        previews = []

        if self.previews is not None:
            for a in self.previews:
                previews.append(a.to_json())

        return {
            'previews': previews
        }



import os
import numbers
from unittest import TestCase
from mailosaur import MailosaurClient
from mailosaur.models import DeviceCreateOptions, MailosaurException


class DevicesTest(TestCase):
    @classmethod
    def setUpClass(self):
        api_key = os.getenv('MAILOSAUR_API_KEY')
        base_url = os.getenv('MAILOSAUR_BASE_URL')

        if api_key is None:
            raise Exception(
                "Missing necessary environment variables - refer to README.md")

        self.client = MailosaurClient(api_key, base_url)

    def test_crud(self):
        device_name = "My test"
        shared_secret = "ONSWG4TFOQYTEMY="

        # Create a new device
        options = DeviceCreateOptions()
        options.name = device_name
        options.shared_secret = shared_secret

        created_device = self.client.devices.create(options)
        self.assertIsNotNone(created_device.id)
        self.assertEqual(device_name, created_device.name)

        # Retrieve an otp via device ID
        otp_result = self.client.devices.otp(created_device.id)
        self.assertEqual(6, len(otp_result.code))

        before = self.client.devices.list()
        self.assertTrue(
            any(x['id'] == created_device.id for x in before.items))

        self.client.devices.delete(created_device.id)

        after = self.client.devices.list()
        self.assertFalse(
            any(x['id'] == created_device.id for x in after.items))

    def test_otp_via_shared_secret(self):
        shared_secret = "ONSWG4TFOQYTEMY="

        otp_result = self.client.devices.otp(shared_secret)
        self.assertEqual(6, len(otp_result.code))



import pytest
import os
import base64
from datetime import datetime, timedelta
from unittest import TestCase
from .mailer import Mailer
from mailosaur import MailosaurClient
from mailosaur.models import Attachment, SearchCriteria, MessageCreateOptions, MessageForwardOptions, MessageReplyOptions, MailosaurException


class EmailsTest(TestCase):
    @classmethod
    def setUpClass(cls):
        api_key = os.getenv('MAILOSAUR_API_KEY')
        base_url = os.getenv('MAILOSAUR_BASE_URL')
        cls.server = os.getenv('MAILOSAUR_SERVER')
        cls.verified_domain = os.getenv('MAILOSAUR_VERIFIED_DOMAIN')

        if (api_key or cls.server) is None:
            raise Exception(
                "Missing necessary environment variables - refer to README.md")

        cls.client = MailosaurClient(api_key, base_url)

        cls.client.messages.delete_all(cls.server)

        Mailer.send_emails(cls.client, cls.server, 5)

        cls.emails = cls.client.messages.list(cls.server).items

    def test_list(self):
        self.assertEqual(5, len(self.emails))

        for email in self.emails:
            self.validate_email_summary(email)

    def test_list_received_after(self):
        past_date = datetime.today() - timedelta(minutes=10)
        past_emails = self.client.messages.list(
            self.server, received_after=past_date).items
        self.assertTrue(len(past_emails) > 0)

        future_emails = self.client.messages.list(
            self.server, received_after=datetime.today() + timedelta(seconds=1)).items
        self.assertEqual(0, len(future_emails))

    def test_get(self):
        host = os.getenv('MAILOSAUR_SMTP_HOST', 'mailosaur.net')
        test_email_address = "wait_for_test@%s.%s" % (self.server, host)

        Mailer.send_email(self.client, self.server, test_email_address)

        criteria = SearchCriteria()
        criteria.sent_to = test_email_address
        email = self.client.messages.get(self.server, criteria)
        self.validate_email(email)

    def test_get_by_id(self):
        email_to_retrieve = self.emails[0]
        email = self.client.messages.get_by_id(email_to_retrieve.id)
        self.validate_email(email)
        self.validate_headers(email)

    def test_get_not_found(self):
        with self.assertRaises(MailosaurException):
            self.client.messages.get_by_id(
                "efe907e9-74ed-4113-a3e0-a3d41d914765")

    def test_search_timeout_errors_suppressed(self):
        criteria = SearchCriteria()
        criteria.sent_from = "neverfound@example.com"
        results = self.client.messages.search(
            self.server, criteria, timeout=1, error_on_timeout=False).items
        self.assertEqual(0, len(results))

    def test_search_no_criteria_error(self):
        with self.assertRaises(MailosaurException):
            self.client.messages.search(self.server, SearchCriteria())

    def test_search_by_sent_from(self):
        target_email = self.emails[1]
        criteria = SearchCriteria()
        criteria.sent_from = target_email.sender[0].email
        results = self.client.messages.search(self.server, criteria).items
        self.assertEqual(1, len(results))
        self.assertEqual(
            target_email.sender[0].email, results[0].sender[0].email)
        self.assertEqual(target_email.subject, results[0].subject)

    def test_search_by_sent_to(self):
        target_email = self.emails[1]
        criteria = SearchCriteria()
        criteria.sent_to = target_email.to[0].email
        results = self.client.messages.search(self.server, criteria).items
        self.assertEqual(1, len(results))
        self.assertEqual(target_email.to[0].email, results[0].to[0].email)
        self.assertEqual(target_email.subject, results[0].subject)

    def test_search_by_body(self):
        target_email = self.emails[1]
        unique_string = target_email.subject[0:10]
        criteria = SearchCriteria()
        criteria.body = "%s html" % (unique_string)
        results = self.client.messages.search(self.server, criteria).items
        self.assertEqual(1, len(results))
        self.assertEqual(target_email.to[0].email, results[0].to[0].email)
        self.assertEqual(target_email.subject, results[0].subject)

    def test_search_by_subject(self):
        target_email = self.emails[1]
        unique_string = target_email.subject[0:10]
        criteria = SearchCriteria()
        criteria.subject = unique_string
        results = self.client.messages.search(self.server, criteria).items
        self.assertEqual(1, len(results))
        self.assertEqual(target_email.to[0].email, results[0].to[0].email)
        self.assertEqual(target_email.subject, results[0].subject)

    def test_search_with_match_all(self):
        target_email = self.emails[1]
        unique_string = target_email.subject[0:10]
        criteria = SearchCriteria()
        criteria.subject = unique_string
        criteria.body = "this is a link"
        criteria.match = "ALL"
        results = self.client.messages.search(self.server, criteria).items
        self.assertEqual(1, len(results))

    def test_search_with_match_any(self):
        target_email = self.emails[1]
        unique_string = target_email.subject[0:10]
        criteria = SearchCriteria()
        criteria.subject = unique_string
        criteria.body = "this is a link"
        criteria.match = "ANY"
        results = self.client.messages.search(self.server, criteria).items
        self.assertEqual(5, len(results))

    def test_search_with_special_characters(self):
        criteria = SearchCriteria()
        criteria.subject = "Search with ellipsis … and emoji 👨🏿‍🚒"
        results = self.client.messages.search(self.server, criteria).items
        self.assertEqual(0, len(results))

    def test_spam_analysis(self):
        target_id = self.emails[0].id
        result = self.client.analysis.spam(target_id)

        for rule in result.spam_filter_results.spam_assassin:
            self.assertIsNotNone(rule.rule)
            self.assertIsNotNone(rule.description)

    def test_deliverability_report(self):
        target_id = self.emails[0].id
        result = self.client.analysis.deliverability(target_id)

        self.assertIsNotNone(result)

        self.assertIsNotNone(result.spf)

        self.assertIsNotNone(result.dmarc)
        for dkim in result.dkim:
            self.assertIsNotNone(dkim)

        self.assertIsNotNone(result.dmarc)

        self.assertIsNotNone(result.block_lists)
        for block_list in result.block_lists:
            self.assertIsNotNone(block_list.id)
            self.assertIsNotNone(block_list.name)

        self.assertIsNotNone(result.content)
        
        self.assertIsNotNone(result.dns_records)
        self.assertIsNotNone(result.dns_records.a)
        self.assertIsNotNone(result.dns_records.mx)
        self.assertIsNotNone(result.dns_records.ptr)

        self.assertIsNotNone(result.spam_assassin)
        self.assertIsNotNone(result.spam_assassin.result)
        self.assertIsNotNone(result.spam_assassin.score)
        for rule in result.spam_assassin.rules:
            self.assertIsNotNone(rule.rule)
            self.assertIsNotNone(rule.description)

    def test_delete(self):
        target_email_id = self.emails[4].id

        self.client.messages.delete(target_email_id)

        # Attempting to delete again should fail
        with self.assertRaises(MailosaurException):
            self.client.messages.delete(target_email_id)

    def test_create_and_send_with_text(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        subject = "New message"
        options = MessageCreateOptions(
            "anything@%s" % (self.verified_domain), True,  subject, "This is a new email")
        message = self.client.messages.create(self.server, options)
        self.assertIsNotNone(message.id)
        self.assertEqual(subject, message.subject)

    def test_create_and_send_with_html(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        subject = "New HTML message"
        options = MessageCreateOptions(
            "anything@%s" % (self.verified_domain), True,  subject, html="<p>This is a new email.</p>")
        message = self.client.messages.create(self.server, options)
        self.assertIsNotNone(message.id)
        self.assertEqual(subject, message.subject)

    def test_create_and_send_with_cc(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        subject = "CC message"
        ccRecipient = "someoneelse@%s" % (self.verified_domain)
        options = MessageCreateOptions(
            "anything@%s" % (self.verified_domain), True,  subject, html="<p>This is a new email.</p>", cc=ccRecipient)
        message = self.client.messages.create(self.server, options)

        self.assertIsNotNone(message.id)
        self.assertEqual(subject, message.subject)
        self.assertEqual(1, len(message.cc))
        self.assertEqual(ccRecipient, message.cc[0].email)

    def test_create_and_send_with_attachment(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        subject = "New message with attachment"

        file = open(os.path.join(os.path.dirname(
            __file__), 'resources', 'cat.png'), 'rb')
        attachment = Attachment()
        attachment.file_name = "cat.png"
        attachment.content = base64.b64encode(file.read()).decode('ascii')
        attachment.content_type = "image/png"
        attachments = [attachment]

        options = MessageCreateOptions("anything@%s" % (self.verified_domain), True,
                                       subject, html="<p>This is a new email.</p>", attachments=attachments)
        message = self.client.messages.create(self.server, options)
        self.assertEqual(1, len(message.attachments))
        file1 = message.attachments[0]
        self.assertIsNotNone(file1.id)
        self.assertIsNotNone(file1.url)
        self.assertEqual(82138, file1.length)
        self.assertEqual("cat.png", file1.file_name)
        self.assertEqual("image/png", file1.content_type)

    def test_forward_with_text(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        body = "Forwarded message"
        options = MessageForwardOptions(
            "anything@%s" % (self.verified_domain), body)
        message = self.client.messages.forward(self.emails[0].id, options)
        self.assertIsNotNone(message.id)
        self.assertTrue(body in message.text.body)

    def test_forward_with_html(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        body = "<p>Forwarded <strong>HTML</strong> message.</p>"
        options = MessageForwardOptions(
            "anything@%s" % (self.verified_domain), html=body)
        message = self.client.messages.forward(self.emails[0].id, options)
        self.assertIsNotNone(message.id)
        self.assertTrue(body in message.html.body)

    def test_forward_with_cc(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        ccRecipient = "someoneelse@%s" % (self.verified_domain)

        body = "<p>Forwarded <strong>HTML</strong> message.</p>"
        options = MessageForwardOptions(
            "forwardcc@%s" % (self.verified_domain), html=body, cc=ccRecipient)
        message = self.client.messages.forward(self.emails[0].id, options)

        self.assertIsNotNone(message.id)
        self.assertTrue(body in message.html.body)
        self.assertEqual(1, len(message.cc))
        self.assertEqual(ccRecipient, message.cc[0].email)

    def test_reply_with_text(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        body = "Reply message body"
        options = MessageReplyOptions(body)
        message = self.client.messages.reply(self.emails[0].id, options)
        self.assertIsNotNone(message.id)
        self.assertTrue(body in message.text.body)

    def test_reply_with_html(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        body = "<p>Reply <strong>HTML</strong> message body.</p>"
        options = MessageReplyOptions(html=body)
        message = self.client.messages.reply(self.emails[0].id, options)
        self.assertIsNotNone(message.id)
        self.assertTrue(body in message.html.body)

    def test_reply_with_cc(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        body = "<p>Reply <strong>HTML</strong> message body.</p>"
        ccRecipient = "someoneelse@%s" % (self.verified_domain)

        options = MessageReplyOptions(html=body, cc=ccRecipient)
        message = self.client.messages.reply(self.emails[0].id, options)

        self.assertIsNotNone(message.id)
        self.assertTrue(body in message.html.body)
        self.assertEqual(1, len(message.cc))
        self.assertEqual(ccRecipient, message.cc[0].email)

    def test_reply_with_attachment(self):
        if self.verified_domain is None:
            pytest.skip("Requires verified domain secret")

        body = "<p>Reply with attachment.</p>"

        file = open(os.path.join(os.path.dirname(
            __file__), 'resources', 'cat.png'), 'rb')
        attachment = Attachment()
        attachment.file_name = "cat.png"
        attachment.content = base64.b64encode(file.read()).decode('ascii')
        attachment.content_type = "image/png"

        options = MessageReplyOptions(html=body, attachments=[attachment])
        message = self.client.messages.reply(self.emails[0].id, options)

        self.assertEqual(1, len(message.attachments))
        file1 = message.attachments[0]
        self.assertIsNotNone(file1.id)
        self.assertIsNotNone(file1.url)
        self.assertEqual(82138, file1.length)
        self.assertEqual("cat.png", file1.file_name)
        self.assertEqual("image/png", file1.content_type)

    def validate_email(self, email):
        self.validate_metadata(email)
        self.validate_attachments(email)
        self.validate_html(email)
        self.validate_text(email)
        self.assertIsNotNone(email.metadata.ehlo)
        self.assertIsNotNone(email.metadata.mail_from)
        self.assertEqual(1, len(email.metadata.rcpt_to))

    def validate_email_summary(self, email):
        self.validate_metadata(email)
        self.assertIsNotNone(email.summary)
        self.assertEqual(2, email.attachments)

    def validate_html(self, email):
        # Html.Body
        self.assertTrue(email.html.body.startswith("<div dir=\"ltr\">"))

        # Html.Links
        self.assertEqual(3, len(email.html.links))
        self.assertEqual("https://mailosaur.com/", email.html.links[0].href)
        self.assertEqual("mailosaur", email.html.links[0].text)
        self.assertEqual("https://mailosaur.com/", email.html.links[1].href)
        self.assertIsNone(email.html.links[1].text)
        self.assertEqual("http://invalid/", email.html.links[2].href)
        self.assertEqual("invalid", email.html.links[2].text)

        # Html.Codes
        self.assertEqual(2, len(email.html.codes))
        self.assertEqual("123456", email.html.codes[0].value)
        self.assertEqual("G3H1Y2", email.html.codes[1].value)

        # Html.Images
        self.assertTrue(email.html.images[1].src.startswith('cid:'))
        self.assertEqual("Inline image 1", email.html.images[1].alt)

    def validate_text(self, email):
        # Text.Body
        self.assertTrue(email.text.body.startswith("this is a test"))

        # Text.Links
        self.assertEqual(2, len(email.text.links))
        self.assertEqual("https://mailosaur.com/", email.text.links[0].href)
        self.assertEqual(email.text.links[0].href, email.text.links[0].text)
        self.assertEqual("https://mailosaur.com/", email.text.links[1].href)
        self.assertEqual(email.text.links[1].href, email.text.links[1].text)

        # Text.Codes
        self.assertEqual(2, len(email.text.codes))
        self.assertEqual("654321", email.text.codes[0].value)
        self.assertEqual("5H0Y2", email.text.codes[1].value)

    def validate_headers(self, email):
        expected_from_header = "%s <%s>" % (
            email.sender[0].name, email.sender[0].email)
        expected_to_header = "%s <%s>" % (email.to[0].name, email.to[0].email)
        headers = email.metadata.headers

        # Invalid python3 syntax
        # print [h for h in headers if h.field.lower() == "from"]

        # self.assertEqual(expected_from_header, [h for h in headers if h.field.lower() == "from"][0].value)
        # self.assertEqual(expected_from_header, [h for h in headers if h.field.lower() == "to"][0].value)
        # self.assertEqual(email.subject, [h for h in headers if h.field.lower() == "subject"][0].value)

    def validate_metadata(self, email):
        self.assertEqual("Email", email.type)
        self.assertEqual(1, len(email.sender))
        self.assertEqual(1, len(email.to))
        self.assertIsNotNone(email.sender[0].email)
        self.assertIsNotNone(email.sender[0].name)
        self.assertIsNotNone(email.to[0].email)
        self.assertIsNotNone(email.to[0].name)
        self.assertIsNotNone(email.subject)
        self.assertIsNotNone(email.server)

        self.assertEqual(datetime.strftime(datetime.now(), '%Y-%m-%d'),
                         datetime.strftime(email.received, '%Y-%m-%d'))

    def validate_attachments(self, email):
        self.assertEqual(2, len(email.attachments))

        file1 = email.attachments[0]
        self.assertIsNotNone(file1.id)
        self.assertIsNotNone(file1.url)
        self.assertEqual(82138, file1.length)
        self.assertEqual("cat.png", file1.file_name)
        self.assertEqual("image/png", file1.content_type)

        file2 = email.attachments[1]
        self.assertIsNotNone(file2.id)
        self.assertIsNotNone(file2.url)
        self.assertEqual(212080, file2.length)
        self.assertEqual("dog.png", file2.file_name)
        self.assertEqual("image/png", file2.content_type)


if __name__ == '__main__':
    unittest.main()



from ..models import UsageAccountLimits
from ..models import UsageTransactionListResult
from ..models import MailosaurException

class UsageOperations(object):
    """UsageOperations operations.
    """

    def __init__(self, session, base_url, handle_http_error):
        self.session = session
        self.base_url = base_url
        self.handle_http_error = handle_http_error

    def limits(self):
        """Retrieve account usage limits.

        Details the current limits and usage for your account.

        :return: UsageAccountLimits
        :rtype: ~mailosaur.models.UsageAccountLimits
        :raises:
        :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/usage/limits" % (self.base_url)
        response = self.session.get(url)
        
        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return UsageAccountLimits(data)

    def transactions(self):
        """Retrieves the last 31 days of transactional usage.

        Details the usage transactions processed by Mailosaur in the last 31 days.

        :return: UsageTransactionListResult
        :rtype: ~mailosaur.models.UsageTransactionListResult
        :raises:
        :class:`MailosaurException<mailosaur.models.MailosaurException>`
        """
        url = "%sapi/usage/transactions" % (self.base_url)
        response = self.session.get(url)
        
        if response.status_code not in [200]:
            self.handle_http_error(response)
            return

        data = response.json()

        return UsageTransactionListResult(data)



class PreviewEmailClient(object):
    """Describes an email client with which email previews can be generated.

    :param id: The unique identifier of the email client.
    :type id: str

    :param name: The display name of the email client.
    :type name: str

    :param platform_group: Whether the platform is desktop, mobile, or web-based.
    :type platform_group: str

    :param platform_type: The type of platform on which the email client is running.
    :type platform_type: str

    :param platform_version: The platform version number.
    :type platform_version: str

    :param can_disable_images: If true, images can be disabled when generating previews.
    :type can_disable_images: bool

    :param status: The current status of the email client.
    :type status: str
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.id = data.get('id', None)
        self.name = data.get('name', None)
        self.platform_group = data.get('platformGroup', None)
        self.platform_type = data.get('platformType', None)
        self.platform_version = data.get('platformVersion', None)
        self.can_disable_images = data.get('canDisableImages', None)
        self.status = data.get('status', None)

    def to_json(self):
        return {
            'id': self.id,
            'name': self.name,
            'platformGroup': self.platform_group,
            'platformType': self.platform_type,
            'platformVersion': self.platform_version,
            'canDisableImages': self.can_disable_images,
            'status': self.status
        }



from .spam_filter_results import SpamFilterResults
from .result_enum import ResultEnum

class EmailAuthenticationResult(object):
    """EmailAuthenticationResult.

    :param result:
    :type result: ~mailosaur.models.ResultEnum
    :param description:
    :type description: string
    :param raw_value:
    :type raw_value: string
    :param tags:
    :type tags: dict
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.result = ResultEnum(data.get('result', None))
        self.description = data.get('description', None)
        self.raw_value = data.get('rawValue', None)
        self.tags = data.get('tags', None)



from .usage_transaction import UsageTransaction

class UsageTransactionListResult(object):
    """UsageTransactionListResult.

    :param items: The individual transactions forming the result.
    :type items: list[~mailosaur.models.UsageTransaction]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.items = [UsageTransaction(i) for i in data.get('items', None)]



from .spam_assassin_rule import SpamAssassinRule

class DnsRecords(object):
    """DnsRecords.

    :param a:
    :type a: list[string]
    :param mx:
    :type mx: list[string]
    :param ptr:
    :type ptr: list[string]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.a = [a for a in data.get('a', None)]
        self.mx = [mx for mx in data.get('mx', None)]
        self.ptr = [ptr for ptr in data.get('ptr', None)]


import os
import numbers
from unittest import TestCase
from mailosaur import MailosaurClient
from mailosaur.models import ServerCreateOptions, MailosaurException


class ServersTest(TestCase):
    @classmethod
    def setUpClass(self):
        api_key = os.getenv('MAILOSAUR_API_KEY')
        base_url = os.getenv('MAILOSAUR_BASE_URL')

        if api_key is None:
            raise Exception(
                "Missing necessary environment variables - refer to README.md")

        self.client = MailosaurClient(api_key, base_url)

    def test_list(self):
        result = self.client.servers.list()
        self.assertTrue(len(result.items) > 1)

    def test_get_not_found(self):
        with self.assertRaises(MailosaurException):
            self.client.servers.get("efe907e9-74ed-4113-a3e0-a3d41d914765")

    def test_crud(self):
        server_name = "My test"

        # Create a new server
        options = ServerCreateOptions(server_name)
        created_server = self.client.servers.create(options)
        self.assertIsNotNone(created_server.id)
        self.assertEqual(server_name, created_server.name)
        self.assertIsInstance(created_server.users, list)
        self.assertIsInstance(created_server.messages, numbers.Number)

        # Retrieve a server and confirm it has expected content
        retrieved_server = self.client.servers.get(created_server.id)
        self.assertEqual(created_server.id, retrieved_server.id)
        self.assertEqual(created_server.name, retrieved_server.name)
        self.assertIsInstance(retrieved_server.users, list)
        self.assertIsInstance(retrieved_server.messages, numbers.Number)

        # Retrieve server password
        password = self.client.servers.get_password(created_server.id)
        self.assertTrue(len(password) >= 8)

        # Update a server and confirm it has changed
        retrieved_server.name += " updated with ellipsis … and emoji 👨🏿‍🚒"
        updated_server = self.client.servers.update(
            retrieved_server.id, retrieved_server)
        self.assertEqual(retrieved_server.id, updated_server.id)
        self.assertEqual(retrieved_server.name, updated_server.name)
        self.assertEqual(retrieved_server.users, updated_server.users)
        self.assertEqual(retrieved_server.messages, updated_server.messages)

        self.client.servers.delete(retrieved_server.id)

        # Attempting to delete again should fail
        with self.assertRaises(MailosaurException):
            self.client.servers.delete(retrieved_server.id)

    def test_failed_create(self):
        with self.assertRaises(MailosaurException) as context:
            options = ServerCreateOptions()
            self.client.servers.create(options)

        ex = context.exception
        self.assertEqual(
            "(name) Servers need a name\r\n", ex.message)
        self.assertEqual("invalid_request", ex.error_type)
        self.assertEqual(400, ex.http_status_code)
        self.assertTrue("{\"type\":" in ex.http_response_body)



"""
    mailosaur.com API library. Basic usage:

    >>> from mailosaur import Mailosaur
    >>> mailbox = Mailosaur("BOX_ID", "YOUR_API_KEY")
    >>> emails = mailbox.get_emails()

    More options at https://mailosaur.com/docs/email/
"""

import uuid
import requests

from .operations.servers_operations import ServersOperations
from .operations.messages_operations import MessagesOperations
from .operations.analysis_operations import AnalysisOperations
from .operations.files_operations import FilesOperations
from .operations.usage_operations import UsageOperations
from .operations.devices_operations import DevicesOperations
from .operations.previews_operations import PreviewsOperations
from .models.mailosaur_exception import MailosaurException


class MailosaurClient(object):
    """ Main class to access Mailosaur.com api. """

    def __init__(self, api_key, base_url="https://mailosaur.com/"):
        """ Pass in your mailbox id and api key to authenticate """
        session = requests.Session()
        session.auth = (api_key, '')
        session.headers.update({'User-Agent': 'mailosaur-python/7.19.0'})

        if base_url is None:
            base_url = "https://mailosaur.com/"

        self.servers = ServersOperations(
            session, base_url, self.handle_http_error)
        self.messages = MessagesOperations(
            session, base_url, self.handle_http_error)
        self.analysis = AnalysisOperations(
            session, base_url, self.handle_http_error)
        self.files = FilesOperations(session, base_url, self.handle_http_error)
        self.usage = UsageOperations(session, base_url, self.handle_http_error)
        self.devices = DevicesOperations(
            session, base_url, self.handle_http_error)
        self.previews = PreviewsOperations(
            session, base_url, self.handle_http_error)

    def handle_http_error(self, response):
        message = ""
        if response.status_code == 400:
            try:
                for error in response.json()['errors']:
                    message += "(%s) %s\r\n" % (
                        error['field'], error['detail'][0]['description'])
            except:
                message = "Request had one or more invalid parameters."

            raise MailosaurException(message,
                                     "invalid_request", response.status_code, response.text)
        elif response.status_code == 401:
            raise MailosaurException("Authentication failed, check your API key.",
                                     "authentication_error", response.status_code, response.text)
        elif response.status_code == 403:
            raise MailosaurException("Insufficient permission to perform that task.",
                                     "permission_error", response.status_code, response.text)
        elif response.status_code == 404:
            raise MailosaurException("Not found, check input parameters.",
                                     "invalid_request", response.status_code, response.text)
        else:
            raise MailosaurException("An API error occurred, see httpResponse for further information.",
                                     "api_error", response.status_code, response.text)



from .spam_assassin_rule import SpamAssassinRule

class SpamFilterResults(object):
    """SpamFilterResults.

    :param spam_assassin:
    :type spam_assassin: list[~mailosaur.models.SpamAssassinRule]
    """

    def __init__(self, data=None):
        if data is None:
            data = {}

        self.spam_assassin = [SpamAssassinRule(i) for i in data.get('spamAssassin', None)]
